function [inv_sigma, designs] = inverse_demand(s,pt,xt,zt, bO, iv,numVars,constrained,included,marketvars) %#ok<STOUT,INUSL>
% Function to estimate inverse demand function as in Compiani (2018)
    % bO should be a (J x 1) matrix where each entry is
    % the order of the polynomials for the demand function for one product.
bernO = bO;
order = bO;
IVbernO = bO+iv;
J = size(s,2);
if size(bO,1) ~=J
    if size(bO,2)==J
        bO=bO';
    else
        error('Error: Matrix of polynomial orders has wrong number of elements')
    end
end

if numel(iv)>1 && size(iv)~=bO
    error('Error: Multiple IV orders submitted but matrix size is wrong')
end 

% --------------------------------------------
% Check normalization
if max(zt,[],'all')>1 || min(zt,[],'all')<0
    error('Error: Instruments are not normalized to be between 0 and 1')
end

if nargin>9
    if max(marketvars,[],'all')>1 || min(zt,[],'all')<0
        error('Error: Market-level variables are not normalized to be between 0 and 1')
    end
end

% --------------------------------------------
% Prep for design matrix and constraints
for j = 1:1:J
    eval(['delta', num2str(j), ' = -1* pt(:,', num2str(j),');']);
end

for xj = 1:1:J
    perm_order = [xj:J, 1:xj-1];
    s_perm = s(:,perm_order);
    % becomes design matrix
    eval(['BERN_x', num2str(xj), ' = [];']);
% Market shares
for j = 1:1:J
     if included(xj,perm_order(j))==1
        eval(['BERN_x', num2str(xj), ' = [BERN_x', num2str(xj), ',bern(s_perm(:,j), bernO(xj,1))];']);
     end
end

% Market characteristics
if nargin >9
    for j = 1:1:size(marketvars,2)
        eval(['BERN_x', num2str(xj), ' = [BERN_x', num2str(xj), ',bern(marketvars(:,j), bernO(xj,1))];']);
    end
end

disp('Done Constructing Design Matrix')
% Define instruments
    cols = logical(included(xj,:)); % only need instruments for included products
    z{xj,1} = zt(:,cols);
    if nargin>9
        z{xj,1} = [zt(:,cols),marketvars];
    end

% --------------------------------------------
% Instruments 
    eval(['A', num2str(xj), ' = [];']);
    ztemp = z{xj,1};
    for zj = 1:1:size(ztemp, 2)
        eval(['A', num2str(xj),' = [A', num2str(xj),', bern(ztemp(:,zj), IVbernO(xj,1))];']);
    end
    eval(['numVars = size(A', num2str(xj),',2)/(IVbernO(xj,1)+1);']); 
    eval(['A', num2str(xj), '= fullInteraction(A', num2str(xj), ', numVars, IVbernO(xj,1));']);
    
disp('Done with instruments')
% --------------------------------------------
% Prep for estimation 
    if nargin<10
        numVars = sum(included(xj,:));
    else
        numVars = sum(included(xj,:)) + size(marketvars,2);
    end
    eval(['design', num2str(xj), ' = fullInteraction(BERN_x', num2str(xj), ',numVars, bernO(xj,1));']);
    eval(['num_params(xj,1) = size(design', num2str(xj),',2);']);

disp('Done with estimation prep')
% --------------------------------------------
% Estimation
end
if constrained==0
    for xj = 1:1:J
        % Unconstrained Estimator
        disp(['UNCONSTRAINED ESTIMATIION BEGINNING'])
        eval(['deltatemp = delta', num2str(xj),';']);
        eval(['Atemp = A',num2str(xj),';']);
        eval(['designtemp = design', num2str(xj), ';']);
        disp(['Number of parameters to estimate is: ', num2str(size(designtemp,2))])
        sigtemp =pinv((designtemp'*Atemp*pinv(Atemp'*Atemp)*Atemp'*designtemp))*designtemp'*Atemp*pinv(Atemp'*Atemp)*Atemp'*deltatemp;
        eval(['sig', num2str(xj),'=sigtemp;']);
        eval(['designs{', num2str(xj),'}=designtemp;']);
        disp(['Estimation for product ', num2str(xj),' is done'])
    end
else
    for xj = 1:1:J
        % Constrained Estimator
        disp(['CONSTRAINED ESTIMATIION BEGINNING'])
        eval(['deltatemp = delta', num2str(xj),';']);
        eval(['Atemp = A',num2str(xj),';']);
        eval(['designtemp = design', num2str(xj), ';']);
        eval(['ApA = transpose(A', num2str(xj), ')*A',num2str(xj),';']);
        if nargin > 9
            numVars = sum(included(xj,:)) + size(marketvars,2);
        else
            numVars = sum(included(xj,:));
        end
        eval(['constMat = makeConstraint(BERN_x', num2str(xj), ', numVars, bernO(xj,1),1,1,1);']);
        if numVars>1 && nargin<10
            for j2 = 2:1:numVars
                eval(['consttemp = makeConstraint(BERN_x', num2str(xj), ', numVars, bernO(xj,1),1,j2,0);']);
                constMat = [constMat;consttemp];
                clear consttemp
            end
        end
        ineqMat = 0*ones(size(constMat,1),1); 
        disp(['Number of parameters to estimate is: ', num2str(size(designtemp,2))])
        ApA = pinv(ApA); 
        obj = @(guess) objective_priceIndex(guess,designtemp,deltatemp, Atemp, ApA);
             % CVX 
             cvx_begin quiet
             variable guess(size(designtemp,2))
             x = (reshape(guess,[numel(guess),1]));
             minimize( obj(x) )
             subject to 
             constMat*x<=ineqMat
             cvx_end            
         sigtemp = x;
         eval(['sig', num2str(xj),'= sigtemp;']);
        disp(['Constrained estimation for product ', num2str(xj),' is done'])
    end
end
eval(['clear A', num2str(xj),' design', num2str(xj),' ApA BERN_x', num2str(xj)])

for j = 1:1:J
    eval(['inv_sigma{j,1} = sig', num2str(j) ';']);
end
end

