function [constMat, constMat2] = makeConstraint(data, numVars, bernO, priceIndex, whichShare, own)
% Data: matrix of non-interacted bernstein polynomials
% numVars: number of variables when priceIndex==1, number of variables per
% product otherwise
% priceIndex: whether price is in index or not
% which market share we care about

p = primes(1000);
pdata = p(1:size(data,2)); % list of primes the size of the data
pdata2 = fullInteraction(pdata, numVars, bernO); % interact primes into design matrix

if nargin>3 && priceIndex==1
    s1p = pdata((bernO+1)*(whichShare-1)+1:(bernO+1)*whichShare); % locations of univariate bernstein polynomial in s1
else
    s1p = pdata(numVars/2*(bernO+1)+1:numVars/2*(bernO+1)+bernO+1); % locations of univariate bernstein polynomial in s1
    s2p = pdata(numVars/2*(bernO+1)+bernO+2:numVars/2*(bernO+1)+2*(bernO+1)); 
    p1p = pdata(1:bernO+1); 
    p2p = pdata(bernO+2:2*(bernO+1));
end

% monotonicity
if numVars>1
    constMat = [];
    for i = 1:1:length(s1p)-1
        t1 = pdata2./s1p(i); % divide data by primes corresponding to terms in polynomial of s
        t2 = pdata2./s1p(i+1);
        for j = 1:1:size(t1,2)
            %isinteger(t1(j))
            if ~isempty(find(t2 ==t1(j),1)) && t1(j)==floor(t1(j)) % find where other terms are the same but s term is one order higher
                z = zeros(1,size(pdata2,2));
                if own ==1 % if own price
                    % assumes s increasing in delta
                    z(j) = 1; % add a 1 at the current location
                    z(t2 ==t1(j)) = -1; % add a -1 where the order of s is one higher
                else
                    z(j) = 1; % add a 1 at the current location
                    z(t2 ==t1(j)) = -1; % add a -1 where the order of s is one higher
                end
                constMat = [constMat; z];
            end
        end
    end
else
    constMat = [];
    i1 = 1;
    i2 = 2;
    for i = 1:1:size(data,2)-1
        z = zeros(1,size(data,2));
        z(i1) = 1;
        z(i2) = -1;
        constMat = [constMat;z]; 
        i1 = i1+1;
        i2 = i2+1;
    end
end
constMat2 = [];
end