function [esep,Jmat,svec,numBadMarkets,all_own] = price_elasticity_priceIndex(inv_sigma, s, p_points, deltas, bO, whichProducts, included, trueS,maxes)
% --------------------------------------------------------------------
% Code takes in estimates from inverse_demand and returns implied price
% elasticity of first good
% --------------------------------------------------------------------

% inv_sigma     -- estimates returned by inverse_demand.m
% s             -- market shares
% p_points      -- vector of prices corresponding to s
% deltas        -- index (excluding xi) at which to evaluate demand
% b0            -- order of Bernstein polynomials
% whichProducts -- [derivative of s_j, with respect to k]
% included      -- matrix indicating which products are substitutes
% trueS         -- evaluate derivatives at realized shares or calculate
%                   counterfactual shares
% maxes         -- object with maxes.maxs = max(s), maxes.mins = min(s),
%                   used if s is small and needs to be rescaled

J = size(s,2);
numBadMarkets = 0;
opts = optimoptions(@fsolve,  'SpecifyObjectiveGradient', true, 'CheckGradient', false, 'Display','none', 'MaxFunctionEvaluations', 10000, 'MaxIterations',50);
if size(bO,1) ~=J
    if size(bO,2)==J
        bO=bO';
    elseif max(size(b0)) ==1
        bO = bO*ones(J,1);
    else
        error('Error: Matrix of polynomial orders has wrong number of elements')
    end
end
bernO=bO;
order = bO;

for j = 1:1:J
    eval(['sig', num2str(j),'=inv_sigma{j,1};'])
end
if ~isempty(maxes)
    maxs = maxes.maxs; 
    mins = maxes.mins;
end

%% Declare prices and shares to evaluate derivatives
if trueS == 0
    for pi = 1:1:size(deltas,1)
        func = @(sj)solve_s_nested_flexible(sj,inv_sigma, deltas(pi,:)',J,order,included,maxes);
        svec(pi,:) = fsolve(func, 0.5*ones(J,1),opts);
    end
else
    svec = s;
end

%% Share Jacobian
clear J_sp_own J_sp_cross
for j1 = 1:1:J
    perm_order = [j1:J, 1:j1-1];
    s_perm = svec(:,perm_order);
    for j2 = 1:1:J 
        if included(j1,j2)==1 % cross-derivatives are zero if not included in j1 demand
            tempmat_s = [];
            for j3 = 1:1:J
                stemp = s_perm(:,j3);
                if included(j1,perm_order(j3))==1 % design matrix only uses included products
                    if j2==perm_order(j3)
                        tempmat_s = [tempmat_s, dbern(stemp, bernO(j1,1))];
                    else
                        tempmat_s = [tempmat_s, bern(stemp, bernO(j1,1))];
                    end
                end
            end
            % Market characteristics
            if nargin>=10
                for i = 1:1:size(marketvars,2)
                    tempmat_s = [tempmat_s, bern(marketvars(:,i),bernO(j1,1))];
                end
            end
            
            tempmat_s = fullInteraction(tempmat_s, sum(included(j1,:)), bernO(j1,1));
            if ~isempty(maxes)
                eval(['dsi', num2str(j1), 'ds', num2str(j2), '= tempmat_s*inv_sigma{j1,1}./ (maxs(1,j2)-mins(1,j2));']);
            else
                eval(['dsi', num2str(j1), 'ds', num2str(j2), '= tempmat_s*inv_sigma{j1,1};']);
            end
        else
            zerotemp = zeros(size(svec,1),1);
            eval(['dsi', num2str(j1), 'ds', num2str(j2), '= zerotemp;']);
        end
    end
end

Jmat = cell(1,size(svec,1));
J_sp = zeros(size(svec(:,1)));
all_own = zeros(size(svec,1),J);
for ii = 1:1:length(dsi1ds1)
    lastwarn('') % Clear last warning message
    J_s=[];
    for j1 = 1:1:J
        J_sj1 = [];
        for j2 = 1:1:J
            eval(['J_sj1 = [J_sj1, dsi', num2str(j1), 'ds', num2str(j2), '(ii)];']);
        end
        J_s = [J_s;J_sj1];
        clear  J_sj1
    end
    temp = -1*inv(J_s);
    if ~isempty(maxes)
        svec2 = svec.*(maxs - mins) + mins; 
    else 
        svec2 = svec;
    end
    all_own(ii,:) = -1*J_s\(p_points(ii,:)./svec2(ii,:))';
    [warnMsg, ~] = lastwarn;
    Jmat{ii} = temp;
    J_sp(ii,1) = temp(whichProducts(1),whichProducts(2));
    if ~isempty(warnMsg)
        numBadMarkets = numBadMarkets+1;
    else
        % disp(['Market ', num2str(ii)])
    end
    clear J_s J_p
end

esep = J_sp.*p_points(:,1)./svec2(:,1); % own-price varying 
if trueS==1
    disp(['There were ', num2str(numBadMarkets),' bad markets out of ' ,num2str(size(s,1))])
end
end
